#pragma once

#include "VivyApplication.hh"
#include "Lib/Log.hh"
#include "Lib/Utils.hh"
#include "Lib/Ass/Style.hh"
#include "Lib/Ass/Line.hh"
#include "Lib/Ass/AssPrivate.hh"

// An ASS file is basically an INI file, but some keys are present multiple
// times (V4+ Styles/Style, Events/Dialogue, Events/Comment).

namespace Vivy::Ass
{
class AssFactory final {
    VIVY_UNMOVABLE_OBJECT(AssFactory)
    VIVY_APP_LOGGABLE_OBJECT(AssFactory, logger)

public:
    enum class Section { ScriptInfo = 1, Styles = 2, Events = 3 };

    using SectionContent = QMap<QString, QVariant>;

private:
    QFile diskStorage;
    SectionContent assInfo{};
    QVector<LinePtr> assLines{};
    QVector<StylePtr> assStyles{};

    StylePtr spareNullStylePtr{ nullptr };

    static inline const QString sectionScriptInfo = "Script Info";
    static inline const QString sectionStyles     = "V4+ Styles";
    static inline const QString sectionEvents     = "Events";
    static inline const QStringList validSections = { sectionEvents, sectionScriptInfo,
                                                      sectionStyles };

    static inline const QStringList intTypeFields{ "PlayResX", "PlayResY", "WrapStyle" };
    static inline const QVector<QPair<QString, QString>> checkedValues{
        { "ScriptType", "v4.00+" },
        { "WrapStyle", "0" },
        // { "YCbCr Matrix", "TV.601" },
        { "ScaledBorderAndShadow", "yes" }
    };

    bool initFromStorage() noexcept;
    bool checkValidity() const noexcept;

public:
    explicit AssFactory(const QString &);
    ~AssFactory() noexcept = default;

    bool hasStyle(const QString &) const noexcept;
    StyleWeakPtr getStyle(const QString &) const noexcept;

    SectionContent getInfoSection() const noexcept;
    void getStyles(QVector<StylePtr> &) const noexcept;
    void getLines(QVector<LinePtr> &) const noexcept;
};
}
