#pragma once

#include "PreCompiledHeaders.hh"
#include "VivyApplication.hh"
#include "Lib/Log.hh"
#include "Lib/Ass/AssPrivate.hh"

namespace Vivy::Ass
{
struct Color {
    static QColor fromString(const QString &) noexcept;
    static QString toString(const QColor &) noexcept;

    static inline const QColor defaultValue = QColor(0, 0, 0, 0);

private:
    Color() = default;
};

class Style final {
    VIVY_APP_LOGGABLE_OBJECT(Style, logger)

    QString styleName;
    QString fontName;
    int fontSize{};

    QColor primaryColor{ Color::defaultValue }, secondaryColor{ Color::defaultValue },
        outlineColor{ Color::defaultValue }, backColor{ Color::defaultValue };

    bool bold{ false }, italic{ false }, underline{ false }, strikeOut{ false };

    float scaleX{ 0.0 }, scaleY{ 0.0 };
    float spacing{ 0.0 }, angle{ 0.0 }, borderStyle{ 0.0 };
    float outline{ 0.0 }, shadow{ 0.0 };

    int alignment{ 2 };                           // Alignement is bottom center
    int marginL{ 0 }, marginR{ 0 }, marginV{ 0 }; // No margin by default
    int encoding{ 1 }; // Encoding is set to 1 in most ASS files by default (at least in Aegisub)

public:
    // Should grab and copy a user-changeable default style
    explicit Style(const Style &) = default;
    explicit Style(const QString &);

    Style &operator=(const Style &) = delete;
    ~Style() noexcept               = default;

    QString getElementName() const noexcept;
    QJsonDocument getProperties() const noexcept;
};
}
