#include "PreCompiledHeaders.hh"
#include "Lib/Ass/Syl.hh"
#include "Lib/Ass/Line.hh"

using namespace Vivy::Ass;

Syl::Syl(Line *const line, const QString &lineString, ConstructMode mode) noexcept
    : content(lineString)
    , styleProperties(line->getStyleProperties())
    , duration(line->getDuration())
    , parentLine(line)
{
    // Will override the `content`, but will be heavy anyway
    if (mode == ConstructMode::ReadAssTags) {
        const int textBegin = lineString.lastIndexOf('}') + 1;
        content             = (textBegin >= lineString.size()) ? "" : lineString.mid(textBegin);
        duration            = getDurationFromString(lineString);
    }
}

quint64
Syl::getDurationFromString(const QString &line) noexcept
{
    QRegularExpression re("\\\\(?:k|K|ko|kf)(\\d+)");
    quint64 duration                   = 0;
    QRegularExpressionMatchIterator it = re.globalMatch(line);

    while (it.hasNext())
        duration += it.next().captured(1).toUInt();

    return duration;
}

QString
Syl::getContent() const noexcept
{
    return content;
}
