#include "PreCompiledHeaders.hh"
#include "Lib/Document//CRTPSubDocument.hh"
#include "Lib/JsonBuilder.hh"

using namespace Vivy;

// Init a ASS sub-document from a file
void
AssSubDocument::initFromPath(const QString &path)
{
    Ass::AssFactory factory(path);
    factory.getStyles(styles);
    factory.getLines(lines);
}

QString
AssSubDocument::getElementName() const noexcept
{
    return "AssSubDocument";
}

const QVector<Ass::LinePtr> &
AssSubDocument::getLines() const noexcept
{
    return lines;
}

const QVector<Ass::StylePtr> &
AssSubDocument::getStyles() const noexcept
{
    return styles;
}

QJsonDocument
AssSubDocument::getProperties() const noexcept
{
    QJsonObject styleObject;
    for (const Ass::StylePtr &style : styles) {
        styleObject.insert(style->getElementName(), style->getProperties().object());
    }

    return JsonBuilder::createOrderedJsonDocument({
        { "File", filePath },
        { "Styles", styleObject },
    });
}
