#include "PreCompiledHeaders.hh"
#include "Lib/Script/CRTPLuaScriptObject.hh"

using namespace Vivy::Script;

int
OptionDeclaration::TOSTRING(lua_State *const L) noexcept
{
    OptionDeclaration *const self = CHECK(L, 1);
    std::string str(className);
    str += std::string_view(" { ");

    for (auto const &[name, opt] : self->options) {
        str += "\"" + name + "\" -> ";
        str += opt.getSignature();
        str += ", ";
    }

    str.resize(str.size() - 2); // Delete the leading ", "
    str += std::string_view(" }");
    lua_pushstring(L, str.c_str());
    return 1;
}

int
OptionDeclaration::addOptionContent(lua_State *const L) noexcept
{
    OptionDeclaration *const self = CHECK(L, 1);
    luaL_checktype(L, 2, LUA_TSTRING);

    const std::string_view optName = CHECK_STRING_VIEW(L, 2);
    const int valueType            = lua_type(L, 3);

    // XXX: Add support for other types!
    if (valueType == LUA_TNUMBER)
        self->options.emplace(optName, lua_tonumber(L, 3));

    else if (valueType == LUA_TSTRING)
        self->options.emplace(optName, CHECK_STRING_VIEW(L, 3));

    else if (valueType == LUA_TBOOLEAN)
        self->options.emplace(optName, static_cast<bool>(lua_toboolean(L, 3)));

    else
        luaL_error(L, "Unsupported type %s", luaL_typename(L, 3));

    LUA_RETURN_NOTHING(L);
}
