#include "PreCompiledHeaders.hh"
#include "UI/DocumentViews/AssLinesModel.hh"

using namespace Vivy;

AssLinesModel::Item::Item(Ass::LineWeakPtr linePtr) noexcept
    : line(linePtr)
{
}

quint64
AssLinesModel::Item::getLineNumber() const noexcept
{
    return 0;
}

QString
AssLinesModel::Item::getLineText() const noexcept
{
    QString ret;
    if (auto ptr = line.lock()) {
        for (const auto &syl : ptr->getContent()) {
            ret.append(syl.getContent());
            ret.append("|");
        }
    }
    ret.remove(ret.size() - 1);
    return ret;
}

bool
AssLinesModel::Item::getIsComment() const noexcept
{
    if (auto ptr = line.lock())
        return ptr->getIsComment();
    return false;
}

QString
AssLinesModel::Item::getLineStyle() const noexcept
{
    if (auto ptr = line.lock()) {
        if (auto style = ptr->getStyle().lock()) {
            return style->getElementName();
        }
    }
    return "";
}

AssLinesModel::AssLinesModel(const QVector<Ass::LinePtr> &lines) noexcept
    : lineRealData(lines)
{
    childs.reserve(lines.count());
    for (const Ass::LinePtr &line : lines)
        childs.append(new Item(Ass::LineWeakPtr{ line }));
}

AssLinesModel::~AssLinesModel() noexcept { qDeleteAll(childs); }

QVariant
AssLinesModel::data(const QModelIndex &index, int role) const noexcept
{
    if (!index.isValid() || index.column() >= Utils::toUnderlying(Item::Field::TotalFieldCount))
        return QVariant();

    const Item *line         = static_cast<const Item *>(index.internalPointer());
    const Item::Field column = static_cast<Item::Field>(index.column());

    // Display role
    if (Qt::DisplayRole == role) {
        switch (column) {
        case Item::Field::Text: return line->getLineText();
        case Item::Field::TotalFieldCount: qFatal("Unreachable");
        }
    }

    return QVariant();
}

bool
AssLinesModel::setData(const QModelIndex &index, const QVariant & /* value */, int role) noexcept
{
    const int col = index.column();
    if (col >= Utils::toUnderlying(Item::Field::TotalFieldCount))
        return false;

    if (Qt::EditRole == role) {
        emit dataChanged(index, index, { Qt::EditRole });
        return true;
    }

    return false;
}

QVariant
AssLinesModel::headerData(int section, Qt::Orientation oriantation, int role) const noexcept
{
    if (Qt::DisplayRole != role)
        return QVariant();

    else if (Qt::Horizontal == oriantation)
        return headers.value(section);

    else if (Qt::Vertical == oriantation)
        return section + 1;

    else
        return QVariant();
}

QModelIndex
AssLinesModel::parent(const QModelIndex &) const noexcept
{
    return QModelIndex();
}

QModelIndex
AssLinesModel::index(int row, int column, const QModelIndex &parent) const noexcept
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();
    else if (row >= childs.size())
        return QModelIndex();
    else
        return createIndex(row, column, childs[row]);
}

int
AssLinesModel::rowCount(const QModelIndex & /* parent */) const noexcept
{
    return childs.size();
}

int
AssLinesModel::columnCount(const QModelIndex & /* parent */) const noexcept
{
    return Utils::toUnderlying(Item::Field::TotalFieldCount);
}

Qt::ItemFlags
AssLinesModel::flags(const QModelIndex &index) const noexcept
{
    [[maybe_unused]] const Item *item = static_cast<const Item *>(index.internalPointer());
    return Qt::ItemNeverHasChildren | Qt::ItemIsSelectable | QAbstractItemModel::flags(index);
}
