#include "PreCompiledHeaders.hh"
#include "UI/DocumentViews/AssLinesView.hh"
#include "VivyApplication.hh"

using namespace Vivy;

AssLinesView::AssLinesView(QAbstractItemModel *model, QWidget *parent) noexcept
    : QTableView(parent)
    , delegate(new Delegate(this))
{
    setMouseTracking(true);
    setItemDelegate(delegate);
    connect(this, &AssLinesView::hoverIndexChanged, delegate,
            &AssLinesView::Delegate::onItemEntered);

    horizontalHeader()->setHighlightSections(false);
    horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
    horizontalHeader()->setStretchLastSection(true);
    horizontalHeader()->hide();
    verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
    verticalHeader()->setVisible(true);

    setSelectionMode(QAbstractItemView::ExtendedSelection);
    setSelectionBehavior(QAbstractItemView::SelectRows);
    setShowGrid(false);

    const QString style    = QStringLiteral("* { font-family: \"FiraCode\"; font-size: 8pt; }");
    const QString hdrStyle = QStringLiteral("::section {"
                                            "  font-family: \"FiraCode\";"
                                            "  font-size:   8pt;"
                                            "}");
    setStyleSheet(style);
    verticalHeader()->setStyleSheet(hdrStyle);
    setModel(model);
}

void
AssLinesView::paintEvent(QPaintEvent *e) noexcept
{
    QTableView::paintEvent(e);
}

void
AssLinesView::mouseMoveEvent(QMouseEvent *event) noexcept
{
    QModelIndex index = indexAt(event->pos());
    emit hoverIndexChanged(index);
}

AssLinesView::Delegate::Delegate(AssLinesView *view, QWidget *parent) noexcept
    : QStyledItemDelegate(parent)
    , tableView(view)
{
}

bool
AssLinesView::Delegate::isHoveredRow(const QStyleOptionViewItem &option,
                                     const QModelIndex &index) const noexcept
{
    bool ret            = false;
    const int gridWidth = tableView->showGrid() ? 1 : 0;

    for (int i = 0; i < index.model()->columnCount(); ++i) {
        QModelIndex idx = index.model()->index(index.row(), i);
        if (const QAbstractItemView *view =
                qobject_cast<const QAbstractItemView *>(option.widget)) {
            QStyleOptionViewItem o;
            o.initFrom(view);
            o.rect = view->visualRect(idx).adjusted(0, 0, gridWidth, gridWidth);
            if (o.rect.contains(view->viewport()->mapFromGlobal(QCursor::pos()))) {
                ret = true;
            }
        }
    }

    return ret;
}

void
AssLinesView::Delegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
                              const QModelIndex &index) const noexcept
{
    QStyleOptionViewItem opt = option;
    if (index.row() == hoveredRow) {
        opt.state |= QStyle::State_MouseOver;
    }
    QStyledItemDelegate::paint(painter, opt, index);
}

void
AssLinesView::Delegate::onItemEntered(const QModelIndex &index) noexcept
{
    hoveredRow = index.row();
}

void
AssLinesView::Delegate::initStyleOption(QStyleOptionViewItem *option,
                                        const QModelIndex &index) const noexcept
{
    QStyledItemDelegate::initStyleOption(option, index);
}
