#include "PreCompiledHeaders.hh"
#include "UI/DocumentViews/TimingScene.hh"
#include "Lib/Utils.hh"

#include <QGraphicsLineItem>
#include <QGraphicsPixmapItem>
#include <QGraphicsView>
#include <QLabel>
#include <QMessageBox>
#include <QMouseEvent>
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QScrollArea>
#include <QScrollBar>
#include <QVBoxLayout>

using namespace Vivy;

TimingScene::TimingScene(QWidget *parent) noexcept
    : QGraphicsScene(parent)
{
}

TimingScene::TimingScene(QImage img_, quint64 soundLength_, QWidget *parent) noexcept
    : QGraphicsScene(parent)
    , img(img_)
    , soundLength(soundLength_)
{
    QPixmap pixmap(QPixmap::fromImage(img));
    backgroundImg = addPixmap(pixmap);
}

void
TimingScene::mousePressEvent(QGraphicsSceneMouseEvent *event) noexcept
{
    QPointF pos        = event->scenePos();
    QGraphicsItem *got = itemAt(pos, QTransform());
    Ass::LinePtr p     = currentLine.lock();

    if (p && (got == nullptr || got == backgroundImg)) [[likely]] {
        // Handle the different cases
        if (timingMode == TimingMode::Line)
            handleMousePressEventLine(event, p);
        else if (timingMode == TimingMode::Syl)
            handleMousePressEventSyl(event, p);
        else if (timingMode == TimingMode::Char)
            handleMousePressEventChar(event, p);
    }

    QGraphicsScene::mousePressEvent(event);
}

void
TimingScene::handleMousePressEventLine(QGraphicsSceneMouseEvent *event, Ass::LinePtr p) noexcept
{
    QPointF pos  = event->scenePos();
    quint64 time = timeFromPos(pos.x());

    if (const auto &btn = event->button(); btn == Qt::LeftButton) {
        p->setStart(time);
    } else if (btn == Qt::RightButton) {
        p->setEnd(time);
    }
}

void
TimingScene::handleMousePressEventSyl(QGraphicsSceneMouseEvent *, Ass::LinePtr) noexcept
{
}

void
TimingScene::handleMousePressEventChar(QGraphicsSceneMouseEvent *, Ass::LinePtr) noexcept
{
}

quint64
TimingScene::timeFromPos(qreal x)
{
    if (const qreal w = width(); x <= 0 || w <= 0) {
        qCritical() << "Try avoid possible divide by zero in the time from position";
        return 0;
    } else {
        return static_cast<quint64>(x) * soundLength / static_cast<quint64>(w);
    }
}

QGraphicsPixmapItem *
TimingScene::bg() noexcept
{
    return backgroundImg;
}
