#pragma once

#include "Lib/Utils.hh"
#include "Lib/Ass/Ass.hh"
#include "UI/DocumentViews/TimingBar.hh"

namespace Vivy
{
class TimingScene final : public QGraphicsScene {
    Q_OBJECT

public:
    enum class TimingMode { Line = 0, Syl = (1 << 1), Char = (1 << 2) };
    Q_DECLARE_FLAGS(TimingModes, TimingMode)

public:
    static inline constexpr QColor startColour = QColor(127, 0, 127);
    static inline constexpr QColor endColour   = QColor(0, 127, 0);

    explicit TimingScene(QWidget *parent = nullptr) noexcept;
    explicit TimingScene(QImage, quint64, QWidget * = nullptr) noexcept;

private:
    QGraphicsPixmapItem *backgroundImg{ nullptr };
    QImage img;
    quint64 soundLength{ 0 };
    Ass::LineWeakPtr currentLine{};
    TimingMode timingMode{ TimingMode::Line };

public:
    QGraphicsPixmapItem *bg() noexcept;
    void mousePressEvent(QGraphicsSceneMouseEvent *event) noexcept override;

private:
    quint64 timeFromPos(qreal x);
    void handleMousePressEventLine(QGraphicsSceneMouseEvent *, Ass::LinePtr) noexcept;
    void handleMousePressEventSyl(QGraphicsSceneMouseEvent *, Ass::LinePtr) noexcept;
    void handleMousePressEventChar(QGraphicsSceneMouseEvent *, Ass::LinePtr) noexcept;

public slots:
};

}

//Only if combinaisons of mode is allowed, it shouldn't be here
//Q_DECLARE_OPERATORS_FOR_FLAGS(TimingScene::TimingModes)
