#include "PreCompiledHeaders.hh"
#include "UI/DocumentViews/TimingView.hh"

using namespace Vivy;

TimingView::TimingView(QImage img, quint64 soundLength, QWidget *parent) noexcept
    : QGraphicsView(parent)
{
    scene = new TimingScene(img, soundLength, this);
    setFixedHeight(img.height());
    setMaximumHeight(img.height());
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Minimum);
    QObject::connect(verticalScrollBar(), &QScrollBar::rangeChanged, this,
                     &TimingView::moveScrollBarToBottom);
    verticalScrollBar()->setValue(verticalScrollBar()->maximum());
    horizontalScrollBar()->setValue(horizontalScrollBar()->minimum());
    setScene(scene);
}

void
TimingView::wheelEvent(QWheelEvent *event) noexcept
{
    horizontalScrollBar()->setValue(horizontalScrollBar()->value() -
                                    event->angleDelta().y() * wheelAngleToScrollRatio);
}

void
TimingView::mousePressEvent(QMouseEvent *event) noexcept
{
    QPoint pos = event->pos();
    pos.rx() += horizontalScrollBar()->value();
    int x = event->pos().x() + horizontalScrollBar()->value();

    QGraphicsItem *got;
    if ((got = scene->itemAt(pos, QTransform())) == nullptr || got == scene->bg()) {
        const int height = static_cast<int>(scene->height());
        scene->addItem(new TimingBar(QLine(x, 0, x, height),
                                     event->button() == Qt::LeftButton ? startColour : endColour));
    }

    QGraphicsView::mousePressEvent(event);
}

void
TimingView::moveScrollBarToBottom(int, int max) noexcept
{
    verticalScrollBar()->setValue(max);
}
