# How to add new third party libraries

1. Check the licence compatibility
2. Extract the archive in this folder
3. Add a `CMakeLists.txt` to be able to compile and link this library
   with Vivy.

Here is a sample `CMakeLists.txt` file for lua:

```cmake
cmake_minimum_required(VERSION 3.5)
project(lua VERSION 5.4.3 LANGUAGES C)

if(WIN32)
    add_definitions(-D_CRT_SECURE_NO_WARNINGS)
endif()

# Lua static library
add_library(lualib STATIC
    lapi.c lcode.c lctype.c ldebug.c ldo.c ldump.c lfunc.c lgc.c
    llex.c lmem.c lobject.c lopcodes.c lparser.c lstate.c lstring.c
    ltable.c ltm.c lundump.c lvm.c lzio.c lauxlib.c lbaselib.c
    lbitlib.c lcorolib.c ldblib.c liolib.c lmathlib.c loslib.c
    lstrlib.c ltablib.c loadlib.c linit.c
)
set_target_properties(lualib PROPERTIES OUTPUT_NAME "lua")

# Lua interpreter
link_directories(${LUA_BINARY_DIR})
add_executable(lua lua.c)
target_link_libraries(lua PRIVATE lualib)
if(UNIX)
    target_link_libraries(lua m)
endif()

# Lua compiler
link_directories(${LUA_BINARY_DIR})
add_executable(luac luac.c)
target_link_libraries(luac lualib)
if(UNIX)
    target_link_libraries(luac m)
endif()
```
